SET PROC TO updres ADDITIVE
RELEASE goUC
PUBLIC goUC
goUC = CREA('xUpdComm')

DEFINE CLASS xUpdComm AS RELATION

  *** Define the Stack as an array property
  PROTECTED aStack[1, 5]

  FUNCTION INIT()
    *** Open the tables if they are not in use
    IF ! USED('saleteam')
      USE saleteam AGAIN IN 0 ALIAS saleteam
    ENDIF
    IF ! USED('salesres')
      USE salesres AGAIN IN 0 ALIAS salesres
    ENDIF
  ENDFUNC

  *** Exposed Main control function
  *** Input parameters are the Starting ID and the Base Value
  *** Returns Success/Failure
  FUNCTION UpdComm( tnStmPK, tnValue)
    LOCAL lnNextID, llRetVal, lnCommDue
    *** Clear the Stack
    DIMENSION THIS.aStack[1, 5]
    STORE 0 TO THIS.aStack
    *** Check parameters
    IF EMPTY( tnStmPK ) OR EMPTY( tnValue )
      *** Must pass a pk and value
      RETURN .F.
    ELSE
      lnNextID = tnStmPK
    ENDIF
    *** Start the Main loop here
    llRetVal = .T.
    DO WHILE ! EMPTY( lnNextID )
      *** Find the record in SaleTeam
      llRetVal = THIS.FindRec( lnNextID, 'saleteam', 'stmpk' )
      IF ! llRetVal
        *** PK does not exist in SaleTeam
        EXIT
      ENDIF
      *** Get Next ID to use from SaleTeam
      lnNextID = saleteam.stm2FKmgr
      *** Now Find the correct Result Record
      llRetVal = THIS.FindRec( saleteam.stmpk, 'salesres', 'resstmfk' )
      IF ! llRetVal
        *** PK does not exist in Results
        EXIT
      ENDIF
      *** Calculate the Commission Payable
      lnCommDue = THIS.CalcComm( saleteam.stmlevel, tnValue )
      *** Store the Data to the stack
      THIS.STORE( lnCommDue )
    ENDDO
    *** If no failures along the way
    IF llRetVal
      llRetVal = THIS.UpdResult()
    ENDIF
    RETURN llRetVal
  ENDFUNC

  *** Wrapper Method for VFP Seek Function
  *** Input parameters are the parameters for calling SEEK()
  *** Returns Success/Failure for the given key
  PROTECTED FUNCTION FindRec( tuKey, tcTable, tcIndex )
    LOCAL llRetVal
    STORE .F. TO llRetVal
    llRetVal = SEEK( tuKey, tcTable, tcIndex )
    RETURN llRetVal
  ENDFUNC

  *** Method to Calculate the Commission Payable
  *** Input parameters are Level and the Base Value
  *** Returns Actual Amount of Commission Payable
  PROTECTED FUNCTION CalcComm( tnLevel, tnValue )
    LOCAL lnCommission
    DO CASE
      CASE tnLevel = 1             && 1% Payable
        lnCommission = tnValue * 0.010
      CASE tnLevel = 2             && 2.5% Payable
        lnCommission = tnValue * 0.025
      CASE tnLevel = 3             && 5% Payable
        lnCommission = tnValue * 0.050
      CASE tnLevel = 4             && 10% Payable
        lnCommission = tnValue * 0.100
      OTHERWISE                    && No Commission is due
        lnCommission = 0
    ENDCASE
    RETURN lnCommission
  ENDFUNC

  *** Method to Store Data to bottom of the Stack
  *** Input parameter is the Commission Increment
  *** No Return Value
  PROTECTED FUNCTION STORE( tnCommDue )
    LOCAL lnRows
    lnRows = ALEN( THIS.aStack, 1)
    lnRows = IIF( EMPTY( THIS.aStack[1,1] ), 1, lnRows + 1 )
    DIMENSION THIS.aStack[lnRows, 5]
    THIS.aStack[lnrows, 1] = salesres.respk
    THIS.aStack[lnrows, 2] = salesres.rescount
    THIS.aStack[lnrows, 3] = salesres.resvalue
    THIS.aStack[lnrows, 4] = THIS.aStack[lnrows, 2] + 1
    THIS.aStack[lnrows, 5] = THIS.aStack[lnrows, 3] + tnCommDue
  ENDFUNC

  *** Method to Update the Result Table
  *** Returns .T./.F.
  PROTECTED FUNCTION UpdResult()
    LOCAL llOk, lnRows, lnCnt
    PRIVATE pnResPk, pnOriCnt, pnOriVal, pnNewCnt, pnNewVal
    *** Start a Transaction
    llOk = .T.
    BEGIN TRANSACTION
    lnRows = ALEN( THIS.aStack, 1 )
    FOR lnCnt = 1 TO lnRows
      STORE 0 TO pnResPk, pnOriCnt, pnOriVal, pnNewCnt, pnNewVal
      THIS.GET( lnCnt )
      *** Find Results Record
      llOk = THIS.FindRec( pnResPk, 'salesres', 'respk' )
      IF ! llOk
        EXIT
      ENDIF
      *** Check Values are unchanged
      IF salesres.rescount = pnOriCnt AND salesres.resvalue = pnOriVal
        *** Replace Data
        REPLACE rescount WITH pnNewCnt, ;
          resvalue WITH pnNewVal IN salesres
        *** Commit Changes
        llOk = TABLEUPDATE( .F., .F., 'salesres' )
        IF ! llOk
          EXIT
        ENDIF
      ELSE
        *** Exit with error if not
        llOk = .F.
        EXIT
      ENDIF
    NEXT
    IF llOk
      END TRANSACTION
    ELSE
      ROLLBACK
    ENDIF
    *** Return Status
    RETURN llOk
  ENDFUNC

  *** Method to Get Data from the Stack
  *** Optional Input Parameter - Specific Line to Retrieve
  *** No Return Value
  PROTECTED FUNCTION GET( tnLineNo )
    *** Retrieve Data From the Stack
    pnResPk  = THIS.aStack[tnLineNo, 1]
    pnOriCnt = THIS.aStack[tnLineNo, 2]
    pnOriVal = THIS.aStack[tnLineNo, 3]
    pnNewCnt = THIS.aStack[tnLineNo, 4]
    pnNewVal = THIS.aStack[tnLineNo, 5]
  ENDFUNC

ENDDEFINE
